{{

______________________________________________________________________________
********************************************************* File starts here ***
Zerocat Chipflasher --- Flash free firmware, kick the Management Engine.

Copyright (C) 2020  Kai Mertens <kmx@posteo.net>

File pccom.spin --- communication framework for other objects

This file is part of Zerocat Chipflasher.

See end of file for terms of use.

******************************************************************************
}}


'   Object Configuration, set via Makefile
'   ======================================
'
'
'   Tell us which baudrate to use, maintain the configuration token:
'
'   * enable : "^# \t "
'   * disable: "^'~ #"
'
'
'~ #   define _CFG_BAUDRATE_38400
'~ #   define _CFG_BAUDRATE_57600
# 	   define _CFG_BAUDRATE_115200
'
'
#   ifdef _CFG_BAUDRATE_38400
#       warning RS232 Baudrate: 38400
#   endif
#   ifdef _CFG_BAUDRATE_57600
#       warning RS232 Baudrate: 57600
#   endif
#   ifdef _CFG_BAUDRATE_115200
#       warning RS232 Baudrate: 115200
#   endif


OBJ

        'objects without VAR block
        pins        : "pins"
        asc         : "ASCII"
        connect     : "connect"
        time        : "time"

        'objects with VAR block

        'include serial driver
        'FullDuplexSerial, drives the serial connection.
        'This file has been downloaded from the Parallax Website.
        'It is part of the AN018 code example, and can be retrieved
        'via firmware2/doc/Makefile.
        fds         : "FullDuplexSerial"


CON     ''
        ''CON Block:

        ''Pins
        PIN_TX  = pins#PIN_TX
        PIN_RX  = pins#PIN_RX

        ''FDS
        FDS_STOP        = time#FDS_STOP
        FDS_MODE        = %0000
        RX_TIMEOUT      = 10

        ''FDS --- Baudrate
#       ifdef _CFG_BAUDRATE_38400
        PC_BAUDRATE = 38_400
#       endif
#       ifdef _CFG_BAUDRATE_57600
        PC_BAUDRATE = 57_600
#       endif
#       ifdef _CFG_BAUDRATE_115200
        PC_BAUDRATE = 115_200
#       endif

        ''Stack
        STACKSIZE   = 20        'longs

        ''ENUM TXMODE_type
        #0, TXMODE_START, TXMODE_CONTINUE, TXMODE_STOP


VAR

        LONG buftxptr
        LONG stack[STACKSIZE]

        BYTE cog2
        BYTE buftx_contains_data
        BYTE lock_txline
        BYTE lock_rxline


PRI     init

        'Initialize Memory

        buftx_contains_data := 0


PUB     start

        ''Get started, launch cogs.
        ''This method blocks until HUB lockbits were available.

        init
        REPEAT UNTIL (lock_txline := LOCKNEW) => 0
        REPEAT UNTIL (lock_rxline := LOCKNEW) => 0
        REPEAT UNTIL fds.start(PIN_RX, PIN_TX, FDS_MODE, PC_BAUDRATE) == TRUE
        fds.rxflush
        REPEAT UNTIL (cog2 := cognew(comm, @stack) + 1) > 0


PUB     stop

        ''Wait until buffered tx data has been sent,
        ''stop cogs, clear and return lock bits.

        REPEAT WHILE buftx_contains_data == 1           'wait until out buffer is empty
        IF cog2                                         'stop comm
            REPEAT UNTIL NOT LOCKSET(lock_txline)
            REPEAT UNTIL NOT LOCKSET(lock_rxline)
            COGSTOP(cog2~ - 1)
            LOCKCLR(lock_rxline)
            LOCKCLR(lock_txline)
            LOCKRET(lock_rxline)
            LOCKRET(lock_txline)

        'stop fds
        time.wait_ms(FDS_STOP, CNT)                     'give serial driver some time to finish work
        fds.stop


PUB     rxflush

        REPEAT UNTIL NOT LOCKSET(lock_rxline)           'lock access to serial rxline
        fds.rxflush
        LOCKCLR(lock_rxline)                            'allow access to serial txline


PUB     rx : r

        REPEAT UNTIL NOT LOCKSET(lock_rxline)           'lock access to serial rxline
        r := fds.rx
        LOCKCLR(lock_rxline)                            'allow access to serial txline


PUB     rxtime (timeout) : r

        REPEAT UNTIL NOT LOCKSET(lock_rxline)           'lock access to serial rxline
        r := fds.rxtime(timeout)
        LOCKCLR(lock_rxline)                            'allow access to serial txline


PUB     tx (b)

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        fds.tx(b)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     dec (b)

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        fds.dec(b)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     pdecx (prefix, b, suffix)

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        fds.str(prefix)
        fds.dec(b)
        fds.str(suffix)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     hex (b, digits)

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        fds.hex(b, digits)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     phex (b, digits)

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        fds.tx("$")
        fds.hex(b, digits)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     str (strptr) : r

        REPEAT UNTIL NOT LOCKSET(lock_txline)           'lock access to serial txline
        r := fds.str(strptr)
        LOCKCLR(lock_txline)                            'allow access to serial txline


PUB     txbuf (srcptr, txmode) : r

        'Returns the reply, received from host, usually ACK.
        'If error, CAN is to be expected.

        'assume positive reply
        r := connect#ACK

        IF txmode <> TXMODE_START
            REPEAT
                r := rx
                IF r <> connect#ENQ
                    QUIT
                'Send same data again, do not update global pointer.
                REPEAT WHILE buftx_contains_data == 1       'wait until tx buffer is empty
                buftx_contains_data := 1

            'Wait until content of tx buffer has been sent,
            'do NOT clear its data lenght record, as this buffer location
            'might be accessed from elsewhere.
            REPEAT WHILE buftx_contains_data == 1           'wait until tx buffer is empty

        IF txmode <> TXMODE_STOP
            IF r == connect#ACK
                'Update global tx buffer pointer to srcptr and set flag, so that a
                'transmission is initiated.
                buftxptr := srcptr                          'update global pointer first,...
                buftx_contains_data := 1                    '...then set flag


PUB     hello

        ''Send a message.

        str(STRING("Hello World!", asc#CR, asc#LF))


PUB     about

        ''Send Object ID and Version Number.

        str(STRING("This is: "))
        str(@DRVID)
        tx(" ")
        str(@DRVVERSION)
        newline


PUB     connection

        ''Display serial driver and current baudrate.

        pdecx(STRING("Full-Duplex Serial Driver v1.2 (Parallax, Inc.), running at "), PC_BAUDRATE, STRING("baud", asc#CR, asc#LF))


PUB     newline

        ''Send @CRLF String.

        str(@CRLF)


PUB     clearline (linelen)

        ''Clear current line and jump to the left.

        tx(asc#CR)
        REPEAT linelen
            tx(asc#SPACE)
        tx(asc#CR)


PRI     comm | index

        'To be executed by extra cog.
        'Manage buffers.

        REPEAT
            'commrx

            'commtx
            IF buftx_contains_data == 1
                IF LOCKSET(lock_txline)
                    fds.wbuf(buftxptr)
                    LOCKCLR(lock_txline)
                    buftx_contains_data := 0


DAT     ''
        ''DAT Block:

                ORG 0

''Strings
CRLF            BYTE asc#CR, asc#LF           'newline byte sequence
NULL            BYTE 0                        'NULL string
DRVID           BYTE "pccom", 0               'Driver ID
DRVVERSION      BYTE "v0.0.0", 0              'Driver Version


{{

******************************************************************************
Terms of Use:

Zerocat Chipflasher is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License as
published by the Free Software Foundation, either version 3 of the
License, or (at your option) any later version.

Zerocat Chipflasher is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License along
with Zerocat Chipflasher.  If not, see <http://www.gnu.org/licenses/>.

************************************************************** End of File ***
}}
